'use strict';

var MyNamespace = MyNamespace || {};

MyNamespace.helpers = {
    isEmpty: function isEmpty(obj) {
        for(var prop in obj) {
            if(obj.hasOwnProperty(prop))
                return false;
        }
        return true;
    }
};

RWADODataAuth.controller('LoginController',
['$scope', '$rootScope', '$location', 'authenticationSvc', 'messageBusSvc', 'locale',
function ($scope, $rootScope, $location, authenticationSvc, messageBusSvc, locale) {

    $scope.login = function () {
        $scope.dataLoading = true;

        // Это делаем на случаей undefined знаечения поля пароль
        if (!$scope.password)
            $scope.password = '';
        
        authenticationSvc.Login($scope.username, $scope.password, function (response) {
            if (response.status == 200) {
                // Успех
                messageBusSvc.AddMessage({ type: 'success', body: locale.getString('messages.authentication_sucess') });
                authenticationSvc.SetCredentials($scope.username, $scope.password);
                $location.path('/');
                $rootScope.$broadcast('refreshLogin');
            } else {
                // Облом
                if (response.status === 401) {
                    var msg = locale.getString('messages.authentication_fail');
                    messageBusSvc.AddMessage({ type: 'error', body:  msg });
                    $scope.error =  msg;
                    $scope.dataLoading = false;
                }
                    
            }
        });
    };
}]);

RWADODataAuth.controller('LogoutController',
['$rootScope', '$location', 'authenticationSvc', 'messageBusSvc',
function ($rootScope, $location, authenticationSvc, messageBusSvc) {
    $rootScope.$broadcast('hideOverlay');
    authenticationSvc.ClearCredentials();
    $location.path('/login');
    $rootScope.$broadcast('refreshLogin');
}]);


RWADODataAuth.controller('UsersController',
['$scope', '$rootScope', '$location', '$compile', '$q', 'usersService',  'rolesService', 'membershipService', 'messageBusSvc', 'spinnerControlSvc', 'DTOptionsBuilder', 'DTColumnBuilder', 'DTDefaultOptions', '$modal', 'dtBuilder', 'locale',
function ($scope, $rootScope, $location, $compile, $q, usersService, rolesService, membershipService, messageBusSvc, spinnerControlSvc, DTOptionsBuilder, DTColumnBuilder, DTDefaultOptions, $modal, dtBuilder, locale) {

    $scope.helpers = MyNamespace.helpers;

    var getUsers = function() {
        return usersService.GetUsersList().then(function(resUsersList) {
            if (resUsersList.error)
                messageBusSvc.AddMessage({ type: 'error', title: locale.getString('messages.permission_denied'), body: locale.getString('admin-users-list.getlist_permission_denied') });

            usersList = resUsersList;
            return resUsersList;
        });  
    };
    
    var usersList = [];

    function dtInstanceCallback(dtInstance) {
        $scope.dtInstance = dtInstance;
    };
    $scope.dtInstanceCallback = dtInstanceCallback;
   
    function DTCeatedRowHandler(row, data, dataIndex) {
        // Recompiling so we can bind Angular directive to the DT
        $compile(angular.element(row).contents())($scope);
    };    
            
    var configureColumnsDefs = function() {
        var res = [];
        var commandColumn = DTColumnBuilder.newColumn(null).withTitle(locale.getString('common.actions')).notSortable()
            .renderWith(actionsColumnHtml);

        res.push(DTColumnBuilder.newColumn('UserID').withTitle(locale.getString('admin-users-list.key')));
        res.push(DTColumnBuilder.newColumn('Login').withTitle(locale.getString('admin-users-list.login')));
        res.push(DTColumnBuilder.newColumn('EMail').withTitle(locale.getString('admin-users-list.email')));
        res.push(DTColumnBuilder.newColumn('LastName').withTitle(locale.getString('admin-users-list.last_name')));
        res.push(DTColumnBuilder.newColumn('FirstName').withTitle(locale.getString('admin-users-list.first_name')));
        res.push(DTColumnBuilder.newColumn('MiddleInitial').withTitle(locale.getString('admin-users-list.middle_initial')));
        res.push(DTColumnBuilder.newColumn('RolesList').withTitle(locale.getString('admin-users-list.user_roles')));
        res.push(commandColumn);        
        
        // Возвращаем разметку  строки комадного столбца
        function actionsColumnHtml(data, type, full, meta) {
            var parameterKey = data.UserID;
            if (data.editLink)
                parameterKey =  '\'' + data.editLink.replace(/\'/g, '\\\'')  + '\'';

            var editMsg = locale.getString('admin-users-list.edit');
            var deleteMsg = locale.getString('admin-users-list.delete');
            var changePwdMsg = locale.getString('admin-users-list.change_password');
            
            var userLogin = "'" + data.Login + "'";

            var dropdown = '<div class="dropdown text-center">'
                        +'  <button class="btn btn-default dropdown-toggle" '
                        +'  type="button" id="dropdownMenu" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true">'
                        +'    <span class="fa fa-arrow-circle-down"></span>'
                        +'  </button>'
                        +'  <ul class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenu">'
                        +'    <li><a ng-click="EditObject(' + parameterKey + ')">' + editMsg + '</a></li>'
                        +'    <li><a ng-click="OpenChangeUserPassWindow(' + userLogin + ')">' + changePwdMsg + '</a></li>'
                        +'    <li><a ng-click="DeleteObject(' + parameterKey + ')">' + deleteMsg + '</a></li>'                                
                        +'  </ul>'
                        +'</div>';

            return dropdown;
        }

        return res;
    }
    configureColumnsDefs();

    var dtOptionsPromise = $q.defer();
    var dtColumnsPromise = $q.defer();

    $scope.dtOptions = dtOptionsPromise.promise;
    $scope.dtColumnDefs = dtColumnsPromise.promise;

    var getOptions = function() {
        locale.ready('admin-users-list')
        .then(function() { return locale.ready('grid'); })
        .then(function() {
            var columns = configureColumnsDefs();
            dtColumnsPromise.resolve(columns);

            var options = dtBuilder.GetBaseDTConfiguration(getUsers, DTCeatedRowHandler);
            dtOptionsPromise.resolve(options);
        });
    };
    getOptions();

    // Открытие модального окна редактирования пароля пользователя
    $scope.OpenChangeUserPassWindow = function(userLogin) {
        var modalInstance = $modal.open({
            templateUrl: '/js/plugins/rwad_auth/views/change-password-modal.html',
            controller: 'ChangeUserPasswordModalCtrl',
            size: 'lg',
            resolve: {
                userLogin: function() {
                    return userLogin;
                }
            }
        });
    };

    // Localize Datatables
    var dtResFile = 'res/localization/datatables-localization.en.json';
    if (locale.getLocale() == "ru-RU")
        dtResFile = 'res/localization/datatables-localization.ru.json';
    DTDefaultOptions.setLanguageSource(dtResFile);

    function FillAvailableRolesList() {
        $scope.RolesList = [];
        rolesService.GetRolesList().then(function(rolesList) {
            var mappedRolesList = rolesList.map(function(role) {
                return {
                    value: role.RoleID,
                    text: role.Name
                };
            });
            $scope.RolesList = mappedRolesList;
        });
    };
    FillAvailableRolesList();
    $scope.rolesSelectorConfig = { };

    function EditObject(objectEditLink) {
        $scope.User = usersList.filter(function(user) {
            return user.UserID == objectEditLink;
        })[0];
  
        membershipService.GetRolesIDsByUser($scope.User.UserID).then(function(userRoleIDsList) {
            $scope.User.SelectedRoles = userRoleIDsList; 
            $scope.isCreateMode = false;
            $scope.toggleSlider();
        });        
    };
    $scope.EditObject = EditObject;

    function CreateObject() {        
        $scope.User = {};
        $scope.isCreateMode = true;
        $scope.toggleSlider();
    };
    $scope.CreateUser = CreateObject;

    // configure edit form slider
    $scope.editFormOpened = false;
    $scope.sliderPosition = 'right';
    if ($scope.sliderPosition == 'right' || $scope.sliderPosition == 'left') {
        // Навесим класс, определяющий margin-top для боковых слайдеров
        $('pageslide .slider-content').addClass('vertical');
    }
        
    $scope.toggleSlider = function() {
        if ($scope.editFormOpened)
            $rootScope.$broadcast('hideOverlay');
        else 
            $rootScope.$broadcast('showOverlay');
                    
        $scope.editFormOpened = !$scope.editFormOpened;
    };

    $scope.reloadData = function() {
        $scope.dtInstance.reloadData();
    }

    $scope.EditSave = function() {
        function editFinish() {
            messageBusSvc.AddMessage({ type: 'success', body: locale.getString('common.data_updated_success') });
            $scope.toggleSlider();
            $scope.reloadData();
        };

        var promise = {};
        if ($scope.User.UserID) {
            promise = usersService.UpdateUser($scope.User).then(function(res) {
                return membershipService.ApplyUserRoles($scope.User.UserID, $scope.User.SelectedRoles);
            }).then(function(res) {
                editFinish();
            });
        } else {
            promise = usersService.CreateUser($scope.User).then(function(res) {
                editFinish();
            });
        }   

        spinnerControlSvc.setLoadingPromise(promise);     
    };

    $scope.DeleteObject = function(userID) {
        var promise = usersService.DeleteUser(userID).then(function(res) {
            messageBusSvc.AddMessage({ type: 'success', body: locale.getString('admin-users-list.user_delete_success') });
            $scope.reloadData();
        });
        spinnerControlSvc.setLoadingPromise(promise);     
    };    
}]);

var compareTo = function() {
    return {
        require: "ngModel",
        scope: {
            otherModelValue: "=compareTo"
        },
        link: function(scope, element, attributes, ngModel) {
             
            ngModel.$validators.compareTo = function(modelValue) {
                return modelValue == scope.otherModelValue;
            };
 
            scope.$watch("otherModelValue", function() {
                ngModel.$validate();
            });
        }
    };
};
 
RWADODataAuth.directive("compareTo", compareTo);

RWADODataAuth.controller('RolesController',
['$scope', '$rootScope', '$location', '$compile',  '$q', 'usersService',  'rolesService', 'membershipService', 'messageBusSvc', 'spinnerControlSvc', 'DTOptionsBuilder', 'DTColumnBuilder', 'DTDefaultOptions', 'dtBuilder', 'locale',
function ($scope, $rootScope, $location, $compile, $q, usersService, rolesService, membershipService, messageBusSvc, spinnerControlSvc, DTOptionsBuilder, DTColumnBuilder, DTDefaultOptions, dtBuilder, locale) {

    $scope.helpers = MyNamespace.helpers;

    var getRoles = function() {
        return rolesService.GetRolesList().then(function(resRolesList) {
            if (resRolesList.error)
                messageBusSvc.AddMessage({ type: 'error', title: locale.getString('common.permission_denied'), body: locale.getString('admin-roles-list.getlist_permission_denied') });
            rolesList = resRolesList;
            return resRolesList;
        });  
    };

    $scope.editFormOpened = false;
    var rolesList = [];

    function dtInstanceCallback(dtInstance) {
        $scope.dtInstance = dtInstance;
    };
    $scope.dtInstanceCallback = dtInstanceCallback;

    function DTCeatedRowHandler(row, data, dataIndex) {
        // Recompiling so we can bind Angular directive to the DT
        $compile(angular.element(row).contents())($scope);
    };
    
    var configureColumnsDefs = function(){
        var res = [];
        var commandColumn = DTColumnBuilder.newColumn(null).withTitle(locale.getString('common.actions')).notSortable()
            .renderWith(actionsColumnHtml);

        res.push(DTColumnBuilder.newColumn('RoleID').withTitle(locale.getString('admin-roles-list.key')));
        res.push(DTColumnBuilder.newColumn('Name').withTitle(locale.getString('admin-roles-list.role_name')));
        res.push(commandColumn);        
        
        // Возвращаем разметку  строки комадного столбца
        function actionsColumnHtml(data, type, full, meta) {
            var parameterKey = data.RoleID;
            if (data.editLink)
                parameterKey =  '\'' + data.editLink.replace(/\'/g, '\\\'')  + '\'';

            var editMsg = locale.getString('admin-roles-list.edit');
            var deleteMsg = locale.getString('admin-roles-list.delete');

            var dropdown = '<div class="dropdown text-center">'
                        +'  <button class="btn btn-default dropdown-toggle" '
                        +'  type="button" id="dropdownMenu" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true">'
                        +'    <span class="fa fa-arrow-circle-down"></span>'
                        +'  </button>'
                        +'  <ul class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenu">'
                        +'    <li><a ng-click="EditObject(' + parameterKey + ')">' + editMsg +' </a></li>'
                        +'    <li><a ng-click="DeleteObject(' + parameterKey + ')">' + deleteMsg + '</a></li>'                                
                        +'  </ul>'
                        +'</div>';

            return dropdown;
        }

        return res;
    }

    var dtOptionsPromise = $q.defer();
    var dtColumnsPromise = $q.defer();

    $scope.dtOptions = dtOptionsPromise.promise;
    $scope.dtColumnDefs = dtColumnsPromise.promise;

    var getOptions = function() {
        locale.ready('admin-roles-list')
        .then(function() { 
            return locale.ready('grid'); 
        })
        .then(function() {
            var columns = configureColumnsDefs();
            dtColumnsPromise.resolve(columns);

            var options = dtBuilder.GetBaseDTConfiguration(getRoles, DTCeatedRowHandler);
            dtOptionsPromise.resolve(options);
        });
    };
    getOptions();

    // Localize Datatables
    var dtResFile = 'res/localization/datatables-localization.en.json';
    if (locale.getLocale() == "ru-RU")
        dtResFile = 'res/localization/datatables-localization.ru.json';
    DTDefaultOptions.setLanguageSource(dtResFile);

    function EditObject(objectEditLink) {
        $scope.Role = rolesList.filter(function(role) {
            return role.RoleID == objectEditLink;
        })[0];

        $scope.isCreateMode = false;
        $scope.ObjectsPermsList = [];
        
        // Сортируем массив полномочий в порядке: R->C->U->D
        function OrderActionsArray(initialArray) {
            var clonedArr = clone(initialArray);
            initialArray[0] = clonedArr.filter(function(item) {
                return item.ActionName === 'GetListOf';
            })[0];
            initialArray[1] = clonedArr.filter(function(item) {
                return item.ActionName === 'Create';
            })[0];
            initialArray[2] = clonedArr.filter(function(item) {
                return item.ActionName === 'Update';
            })[0];
            initialArray[3] = clonedArr.filter(function(item) {
                return item.ActionName === 'Delete';
            })[0];

            return initialArray;
        };

        rolesService.GetRolePermissions($scope.Role.RoleID).then(function(permissionsDefList) {
            var currentTablePerms = [];            
            var resList = [];
            var adminTableNames = ['RWADOData_User', 'RWADOData_Role', 'RWADOData_Permission', 'RWADOData_UserRole', 'RWADOData_PermissionRole', 
                'User', 'Role', 'Permission', 'UserRole', 'PermissionRole'];
            if(permissionsDefList.length > 0) {
                var curTableName = permissionsDefList[0].TableName;
                    angular.forEach(permissionsDefList, function(item) {
                    if (curTableName !== item.TableName) {
                        var isAdminObject = adminTableNames.indexOf(curTableName) > -1;
                        var orderedArray = OrderActionsArray(currentTablePerms);
                        resList.push({
                            TableName: curTableName,
                            PermsList: orderedArray,
                            GroupFlagsVal: false,
                            IsAdminObject: isAdminObject  
                        });
                        curTableName = item.TableName;
                        currentTablePerms = [];
                    };
                    currentTablePerms.push({
                        ActionName: item.ActionName,
                        PermissionID: item.PermissionID,
                        IsEnabled: item.IsEnabled
                    });
                                        
                });
                
                var isAdminObject = adminTableNames.indexOf(curTableName) > -1;
                var orderedArray = OrderActionsArray(currentTablePerms);
                resList.push({
                    TableName: curTableName,
                    PermsList:  orderedArray,
                    GroupFlagsVal: false,
                    IsAdminObject: isAdminObject
                });
            }

            $scope.ObjectsPermsList = resList;
            $scope.toggleSlider();
        });        
    };
    $scope.EditObject = EditObject;
    $scope.ChangeEnabledForObject = function(tableName) {
        tableName = tableName.tableName;
        var targetObjectsPerms = $scope.ObjectsPermsList.filter(function(item) {
            return item.TableName == tableName;
        })[0];
        angular.forEach(targetObjectsPerms.PermsList, function(item) {
            item.IsEnabled = targetObjectsPerms.GroupFlagsVal;
        });
    };

    $scope.IsCheckAllPerms = false;
    $scope.CheckAllPermsChange = function(Params) {
        var resVal = Params.Value;
        angular.forEach($scope.ObjectsPermsList, function(item) {
            if (!item.IsAdminObject) {
                item.GroupFlagsVal =  resVal;
                angular.forEach(item.PermsList, function(PermValue) {
                    PermValue.IsEnabled = resVal;
                });    
            }        
        });
    };

    function CreateObject() {        
        $scope.Role = {};
        $scope.isCreateMode = true;
        $scope.toggleSlider();
    };
    $scope.CreateRole = CreateObject;

    // configure edit form slider
    $scope.editFormOpened = false;
    $scope.sliderPosition = 'right';
    if ($scope.sliderPosition == 'right' || $scope.sliderPosition == 'left') {
        // Навесим класс, определяющий margin-bottom для боковых слайдеров
        $('pageslide .slider-content').addClass('vertical');
    }
        
    $scope.toggleSlider = function() {
        if ($scope.editFormOpened)
            $rootScope.$broadcast('hideOverlay');
        else 
            $rootScope.$broadcast('showOverlay');
                    
        $scope.editFormOpened = !$scope.editFormOpened;
    };

    $scope.reloadData = function() {
        $scope.dtInstance.reloadData();
    }

    $scope.EditSave = function() {
        function editFinish() {
            messageBusSvc.AddMessage({ type: 'success', body: locale.getString('common.data_updated_success') });
            $scope.toggleSlider();            
            $scope.reloadData();
        };

        var promise = {}; 
        if ($scope.Role.RoleID) {
            promise = rolesService.UpdateRole($scope.Role).then(function(res) {
                var enabledPermissionsIdList = [];
                angular.forEach($scope.ObjectsPermsList, function(item) {
                    angular.forEach(item.PermsList, function(PermValue) {
                        if (PermValue.IsEnabled)
                            enabledPermissionsIdList.push(PermValue.PermissionID); 
                    }); 
                });
                
                return rolesService.ApplyRolePermissions($scope.Role.RoleID, enabledPermissionsIdList);
            }).then(function() {
                editFinish();
            });
        } else {
            promise = rolesService.CreateRole($scope.Role).then(function(res) {
                $scope.Role.RoleID = res;
                editFinish();
            });
        }   

        spinnerControlSvc.setLoadingPromise(promise);     
    };

    $scope.DeleteObject = function(roleID) {
        var promise = rolesService.DeleteRole(roleID).then(function(res) {
            messageBusSvc.AddMessage({ type: 'success', body: locale.getString('admin-roles-list.role_delete_success') });
            $scope.reloadData();
        });
        spinnerControlSvc.setLoadingPromise(promise);
    };    
}]);

RWADODataAuth.controller('ChangeUserPasswordModalCtrl',
['$scope', '$modalInstance', 'spinnerControlSvc', 'usersService', 'messageBusSvc', 'locale', 'userLogin',
function ($scope, $modalInstance, spinnerControlSvc, usersService, messageBusSvc, locale, userLogin) {

    $scope.helpers = MyNamespace.helpers;

    $scope.User = {
        Login: userLogin,
        Password: '',
        ConfirmPassword: ''
    };

    $scope.changePassword = function () {
        var promise = usersService.UpdateUserPassword($scope.User.Login, $scope.User.Password).then(function(res) {
            if (!res.error) {
                messageBusSvc.AddMessage({ type: 'success', body: locale.getString('change-password-modal.password_changed_success') });
                $modalInstance.close();    
            } else {
                messageBusSvc.AddMessage({ type: 'error', body: locale.getString('change-password-modal.password_changed_error', { errorText: res.message }) });
            }            
        });
        spinnerControlSvc.setLoadingPromise(promise);
    };

    $scope.modalCancel = function () {
        $modalInstance.dismiss('cancel');
    };
}]);